=== modified file 'NEWS'
--- NEWS	2009-08-26 06:49:49 +0000
+++ NEWS	2009-08-26 09:14:13 +0000
@@ -32,6 +32,12 @@
 Bug Fixes
 *********
 
+* Fetches were being requested in 'groupcompress' order, but weren't
+  recombining the groups. Thus they would 'fragment' to get the correct
+  order, but not 'recombine' to acutally benefit from it. Until we get
+  recombining to work, switching to 'unordered' fetches avoids the
+  fragmentation. (John Arbash Meinel, #402645)
+
 * Fix a pycurl related test failure on karmic by recognizing an error
   raised by newer versions of pycurl.
   (Vincent Ladeuil, #306264)
@@ -101,6 +107,24 @@
 Testing
 *******
 
+bzr 1.18.1 NOT RELEASED YET
+###########################
+
+Bug Fixes
+*********
+
+* Fixed a problem where using content filtering and especially end-of-line
+  conversion will commit too many copies a file.
+  (Martin Pool, #415508)
+
+API Changes
+***********
+
+* ``Tree.path_content_summary`` may return a size of None, when called on
+  a tree with content filtering where the size of the canonical form
+  cannot be cheaply determined.  (Martin Pool)
+
+
 bzr 1.18
 ########
 
@@ -216,6 +240,7 @@
 ###########
 
 :Codename: little traveller
+:1.18:    2009-08-20
 :1.18rc1: 2009-08-10
 
 This release of Bazaar marches on towards the 2.0 release in which the 2a
@@ -229,6 +254,7 @@
 with bugfix-only releases based on it continuing for at least six months
 or until the following stable release.
 
+There are no changes from 1.18rc1 to 1.18.
 
 New Features
 ************

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2009-08-26 03:20:32 +0000
+++ bzrlib/commit.py	2009-08-26 05:38:16 +0000
@@ -805,10 +805,11 @@
                 # _update_builder_with_changes.
                 continue
             content_summary = self.work_tree.path_content_summary(path)
+            kind = content_summary[0]
             # Note that when a filter of specific files is given, we must only
             # skip/record deleted files matching that filter.
             if not specific_files or is_inside_any(specific_files, path):
-                if content_summary[0] == 'missing':
+                if kind == 'missing':
                     if not deleted_paths:
                         # path won't have been split yet.
                         path_segments = splitpath(path)
@@ -821,23 +822,20 @@
                     continue
             # TODO: have the builder do the nested commit just-in-time IF and
             # only if needed.
-            if content_summary[0] == 'tree-reference':
+            if kind == 'tree-reference':
                 # enforce repository nested tree policy.
                 if (not self.work_tree.supports_tree_reference() or
                     # repository does not support it either.
                     not self.branch.repository._format.supports_tree_reference):
-                    content_summary = ('directory',) + content_summary[1:]
-            kind = content_summary[0]
-            # TODO: specific_files filtering before nested tree processing
-            if kind == 'tree-reference':
-                if self.recursive == 'down':
+                    kind = 'directory'
+                    content_summary = (kind, None, None, None)
+                elif self.recursive == 'down':
                     nested_revision_id = self._commit_nested_tree(
                         file_id, path)
-                    content_summary = content_summary[:3] + (
-                        nested_revision_id,)
+                    content_summary = (kind, None, None, nested_revision_id)
                 else:
-                    content_summary = content_summary[:3] + (
-                        self.work_tree.get_reference_revision(file_id),)
+                    nested_revision_id = self.work_tree.get_reference_revision(file_id)
+                    content_summary = (kind, None, None, nested_revision_id)
 
             # Record an entry for this item
             # Note: I don't particularly want to have the existing_ie

=== modified file 'bzrlib/repofmt/groupcompress_repo.py'
--- bzrlib/repofmt/groupcompress_repo.py	2009-08-18 05:18:52 +0000
+++ bzrlib/repofmt/groupcompress_repo.py	2009-08-24 19:34:13 +0000
@@ -932,7 +932,7 @@
         super(GroupCHKStreamSource, self).__init__(from_repository, to_format)
         self._revision_keys = None
         self._text_keys = None
-        self._text_fetch_order = 'groupcompress'
+        # self._text_fetch_order = 'unordered'
         self._chk_id_roots = None
         self._chk_p_id_roots = None
 
@@ -949,7 +949,7 @@
             p_id_roots_set = set()
             source_vf = self.from_repository.inventories
             stream = source_vf.get_record_stream(inventory_keys,
-                                                 'groupcompress', True)
+                                                 'unordered', True)
             for record in stream:
                 if record.storage_kind == 'absent':
                     if allow_absent:

=== modified file 'bzrlib/repository.py'
--- bzrlib/repository.py	2009-08-26 03:20:32 +0000
+++ bzrlib/repository.py	2009-08-26 07:40:38 +0000
@@ -465,9 +465,9 @@
             if content_summary[2] is None:
                 raise ValueError("Files must not have executable = None")
             if not store:
-                if (# if the file length changed we have to store:
-                    parent_entry.text_size != content_summary[1] or
-                    # if the exec bit has changed we have to store:
+                # We can't trust a check of the file length because of content
+                # filtering...
+                if (# if the exec bit has changed we have to store:
                     parent_entry.executable != content_summary[2]):
                     store = True
                 elif parent_entry.text_sha1 == content_summary[3]:
@@ -540,6 +540,9 @@
                 ie.revision = parent_entry.revision
                 return self._get_delta(ie, basis_inv, path), False, None
             ie.reference_revision = content_summary[3]
+            if ie.reference_revision is None:
+                raise AssertionError("invalid content_summary for nested tree: %r"
+                    % (content_summary,))
             self._add_text_to_weave(ie.file_id, '', heads, None)
         else:
             raise NotImplementedError('unknown kind')

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2009-08-20 05:05:59 +0000
+++ bzrlib/tests/__init__.py	2009-08-24 05:23:11 +0000
@@ -3207,6 +3207,7 @@
              starting_with=None,
              runner_class=None,
              suite_decorators=None,
+             stream=None,
              ):
     """Run the whole test suite under the enhanced runner"""
     # XXX: Very ugly way to do this...
@@ -3230,9 +3231,14 @@
         else:
             keep_only = load_test_id_list(load_list)
         if test_suite_factory is None:
+            # Reduce loading time by loading modules based on the starting_with
+            # patterns.
             suite = test_suite(keep_only, starting_with)
         else:
             suite = test_suite_factory()
+        if starting_with:
+            # But always filter as requested.
+            suite = filter_suite_by_id_startswith(suite, starting_with)
         return run_suite(suite, 'testbzr', verbose=verbose, pattern=pattern,
                      stop_on_failure=stop_on_failure,
                      transport=transport,
@@ -3245,6 +3251,7 @@
                      strict=strict,
                      runner_class=runner_class,
                      suite_decorators=suite_decorators,
+                     stream=stream,
                      )
     finally:
         default_transport = old_transport
@@ -3683,9 +3690,6 @@
             reload(sys)
             sys.setdefaultencoding(default_encoding)
 
-    if starting_with:
-        suite = filter_suite_by_id_startswith(suite, starting_with)
-
     if keep_only is not None:
         # Now that the referred modules have loaded their tests, keep only the
         # requested ones.

=== modified file 'bzrlib/tests/blackbox/test_selftest.py'
--- bzrlib/tests/blackbox/test_selftest.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/blackbox/test_selftest.py	2009-08-24 05:23:11 +0000
@@ -16,474 +16,111 @@
 
 """UI tests for the test framework."""
 
-from cStringIO import StringIO
-import os
-import re
-import signal
-import sys
-import unittest
-
-import bzrlib
+import bzrlib.transport
 from bzrlib import (
-    osutils,
+    benchmarks,
+    tests,
     )
 from bzrlib.errors import ParamikoNotPresent
 from bzrlib.tests import (
                           SubUnitFeature,
                           TestCase,
                           TestCaseInTempDir,
-                          TestCaseWithMemoryTransport,
-                          TestCaseWithTransport,
-                          TestUIFactory,
                           TestSkipped,
                           )
-from bzrlib.tests.blackbox import ExternalBase
-
-
-class TestOptions(TestCase):
-
-    current_test = None
+
+
+class SelfTestPatch:
+
+    def get_params_passed_to_core(self, cmdline):
+        params = []
+        def selftest(*args, **kwargs):
+            """Capture the arguments selftest was run with."""
+            params.append((args, kwargs))
+            return True
+        # Yes this prevents using threads to run the test suite in parallel,
+        # however we don't have a clean dependency injector for commands, 
+        # and even if we did - we'd still be testing that the glue is wired
+        # up correctly. XXX: TODO: Solve this testing problem.
+        original_selftest = tests.selftest
+        tests.selftest = selftest
+        try:
+            self.run_bzr(cmdline)
+            return params[0]
+        finally:
+            tests.selftest = original_selftest
+
+
+class TestOptionsWritingToDisk(TestCaseInTempDir, SelfTestPatch):
+
+    def test_benchmark_runs_benchmark_tests(self):
+        """selftest --benchmark should change the suite factory."""
+        params = self.get_params_passed_to_core('selftest --benchmark')
+        self.assertEqual(benchmarks.test_suite,
+            params[1]['test_suite_factory'])
+        self.assertNotEqual(None, params[1]['bench_history'])
+        benchfile = open(".perf_history", "rt")
+        try:
+            lines = benchfile.readlines()
+        finally:
+            benchfile.close()
+        # Because we don't run the actual test code no output is made to the
+        # file.
+        self.assertEqual(0, len(lines))
+
+
+class TestOptions(TestCase, SelfTestPatch):
+
+    def test_load_list(self):
+        params = self.get_params_passed_to_core('selftest --load-list foo')
+        self.assertEqual('foo', params[1]['load_list'])
 
     def test_transport_set_to_sftp(self):
-        # test the --transport option has taken effect from within the
-        # test_transport test
+        # Test that we can pass a transport to the selftest core - sftp
+        # version.
         try:
             import bzrlib.transport.sftp
         except ParamikoNotPresent:
             raise TestSkipped("Paramiko not present")
-        if TestOptions.current_test != "test_transport_set_to_sftp":
-            return
+        params = self.get_params_passed_to_core('selftest --transport=sftp')
         self.assertEqual(bzrlib.transport.sftp.SFTPAbsoluteServer,
-                         bzrlib.tests.default_transport)
+            params[1]["transport"])
 
     def test_transport_set_to_memory(self):
-        # test the --transport option has taken effect from within the
-        # test_transport test
+        # Test that we can pass a transport to the selftest core - memory
+        # version.
         import bzrlib.transport.memory
-        if TestOptions.current_test != "test_transport_set_to_memory":
-            return
+        params = self.get_params_passed_to_core('selftest --transport=memory')
         self.assertEqual(bzrlib.transport.memory.MemoryServer,
-                         bzrlib.tests.default_transport)
-
-    def test_transport(self):
-        # test that --transport=sftp works
-        try:
-            import bzrlib.transport.sftp
-        except ParamikoNotPresent:
-            raise TestSkipped("Paramiko not present")
-        old_transport = bzrlib.tests.default_transport
-        old_root = TestCaseWithMemoryTransport.TEST_ROOT
-        TestCaseWithMemoryTransport.TEST_ROOT = None
-        try:
-            TestOptions.current_test = "test_transport_set_to_sftp"
-            stdout = self.run_bzr(
-                'selftest --transport=sftp test_transport_set_to_sftp')[0]
-            self.assertContainsRe(stdout, 'Ran 1 test')
-            self.assertEqual(old_transport, bzrlib.tests.default_transport)
-
-            TestOptions.current_test = "test_transport_set_to_memory"
-            stdout = self.run_bzr(
-                'selftest --transport=memory test_transport_set_to_memory')[0]
-            self.assertContainsRe(stdout, 'Ran 1 test')
-            self.assertEqual(old_transport, bzrlib.tests.default_transport)
-        finally:
-            bzrlib.tests.default_transport = old_transport
-            TestOptions.current_test = None
-            TestCaseWithMemoryTransport.TEST_ROOT = old_root
+            params[1]["transport"])
+
+    def test_parameters_passed_to_core(self):
+        params = self.get_params_passed_to_core('selftest --list-only')
+        self.assertTrue("list_only" in params[1])
+        params = self.get_params_passed_to_core('selftest --list-only selftest')
+        self.assertTrue("list_only" in params[1])
+        params = self.get_params_passed_to_core(['selftest', '--list-only',
+            '--exclude', 'selftest'])
+        self.assertTrue("list_only" in params[1])
+        params = self.get_params_passed_to_core(['selftest', '--list-only',
+            'selftest', '--randomize', 'now'])
+        self.assertSubset(["list_only", "random_seed"], params[1])
+
+    def test_starting_with(self):
+        params = self.get_params_passed_to_core('selftest --starting-with foo')
+        self.assertEqual(['foo'], params[1]['starting_with'])
+
+    def test_starting_with_multiple_argument(self):
+        params = self.get_params_passed_to_core(
+            'selftest --starting-with foo --starting-with bar')
+        self.assertEqual(['foo', 'bar'], params[1]['starting_with'])
 
     def test_subunit(self):
-        """Passing --subunit results in subunit output."""
         self.requireFeature(SubUnitFeature)
-        from subunit import ProtocolTestCase
-        stdout = self.run_bzr(
-            'selftest --subunit --no-plugins '
-            'tests.test_selftest.SelftestTests.test_import_tests')[0]
-        stream = StringIO(str(stdout))
-        test = ProtocolTestCase(stream)
-        result = unittest.TestResult()
-        test.run(result)
-        self.assertEqual(1, result.testsRun)
-
-
-class TestRunBzr(ExternalBase):
-
-    def _run_bzr_core(self, argv, retcode=0, encoding=None, stdin=None,
-                         working_dir=None):
-        """Override _run_bzr_core to test how it is invoked by run_bzr.
-
-        Attempts to run bzr from inside this class don't actually run it.
-
-        We test how run_bzr actually invokes bzr in another location.
-        Here we only need to test that it is run_bzr passes the right
-        parameters to run_bzr.
-        """
-        self.argv = list(argv)
-        self.retcode = retcode
-        self.encoding = encoding
-        self.stdin = stdin
-        self.working_dir = working_dir
-        return '', ''
-
-    def test_encoding(self):
-        """Test that run_bzr passes encoding to _run_bzr_core"""
-        self.run_bzr('foo bar')
-        self.assertEqual(None, self.encoding)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr('foo bar', encoding='baz')
-        self.assertEqual('baz', self.encoding)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-    def test_retcode(self):
-        """Test that run_bzr passes retcode to _run_bzr_core"""
-        # Default is retcode == 0
-        self.run_bzr('foo bar')
-        self.assertEqual(0, self.retcode)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr('foo bar', retcode=1)
-        self.assertEqual(1, self.retcode)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr('foo bar', retcode=None)
-        self.assertEqual(None, self.retcode)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr(['foo', 'bar'], retcode=3)
-        self.assertEqual(3, self.retcode)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-    def test_stdin(self):
-        # test that the stdin keyword to run_bzr is passed through to
-        # _run_bzr_core as-is. We do this by overriding
-        # _run_bzr_core in this class, and then calling run_bzr,
-        # which is a convenience function for _run_bzr_core, so
-        # should invoke it.
-        self.run_bzr('foo bar', stdin='gam')
-        self.assertEqual('gam', self.stdin)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr('foo bar', stdin='zippy')
-        self.assertEqual('zippy', self.stdin)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-    def test_working_dir(self):
-        """Test that run_bzr passes working_dir to _run_bzr_core"""
-        self.run_bzr('foo bar')
-        self.assertEqual(None, self.working_dir)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-        self.run_bzr('foo bar', working_dir='baz')
-        self.assertEqual('baz', self.working_dir)
-        self.assertEqual(['foo', 'bar'], self.argv)
-
-    def test_reject_extra_keyword_arguments(self):
-        self.assertRaises(TypeError, self.run_bzr, "foo bar",
-                          error_regex=['error message'])
-
-
-class TestBenchmarkTests(TestCaseWithTransport):
-
-    def test_benchmark_runs_benchmark_tests(self):
-        """bzr selftest --benchmark should not run the default test suite."""
-        # We test this by passing a regression test name to --benchmark, which
-        # should result in 0 tests run.
-        old_root = TestCaseWithMemoryTransport.TEST_ROOT
-        try:
-            TestCaseWithMemoryTransport.TEST_ROOT = None
-            out, err = self.run_bzr('selftest --benchmark'
-                                    ' per_workingtree')
-        finally:
-            TestCaseWithMemoryTransport.TEST_ROOT = old_root
-        self.assertContainsRe(out, 'Ran 0 tests.*\n\nOK')
-        self.assertContainsRe(out, 'tests passed\n')
-        benchfile = open(".perf_history", "rt")
-        try:
-            lines = benchfile.readlines()
-        finally:
-            benchfile.close()
-        self.assertEqual(1, len(lines))
-        self.assertContainsRe(lines[0], "--date [0-9.]+")
-
-
-class TestRunBzrCaptured(ExternalBase):
-
-    def apply_redirected(self, stdin=None, stdout=None, stderr=None,
-                         a_callable=None, *args, **kwargs):
-        self.stdin = stdin
-        self.factory_stdin = getattr(bzrlib.ui.ui_factory, "stdin", None)
-        self.factory = bzrlib.ui.ui_factory
-        self.working_dir = osutils.getcwd()
-        stdout.write('foo\n')
-        stderr.write('bar\n')
-        return 0
-
-    def test_stdin(self):
-        # test that the stdin keyword to _run_bzr_core is passed through to
-        # apply_redirected as a StringIO. We do this by overriding
-        # apply_redirected in this class, and then calling _run_bzr_core,
-        # which calls apply_redirected.
-        self.run_bzr(['foo', 'bar'], stdin='gam')
-        self.assertEqual('gam', self.stdin.read())
-        self.assertTrue(self.stdin is self.factory_stdin)
-        self.run_bzr(['foo', 'bar'], stdin='zippy')
-        self.assertEqual('zippy', self.stdin.read())
-        self.assertTrue(self.stdin is self.factory_stdin)
-
-    def test_ui_factory(self):
-        # each invocation of self.run_bzr should get its
-        # own UI factory, which is an instance of TestUIFactory,
-        # with stdin, stdout and stderr attached to the stdin,
-        # stdout and stderr of the invoked run_bzr
-        current_factory = bzrlib.ui.ui_factory
-        self.run_bzr(['foo'])
-        self.failIf(current_factory is self.factory)
-        self.assertNotEqual(sys.stdout, self.factory.stdout)
-        self.assertNotEqual(sys.stderr, self.factory.stderr)
-        self.assertEqual('foo\n', self.factory.stdout.getvalue())
-        self.assertEqual('bar\n', self.factory.stderr.getvalue())
-        self.assertIsInstance(self.factory, TestUIFactory)
-
-    def test_working_dir(self):
-        self.build_tree(['one/', 'two/'])
-        cwd = osutils.getcwd()
-
-        # Default is to work in the current directory
-        self.run_bzr(['foo', 'bar'])
-        self.assertEqual(cwd, self.working_dir)
-
-        self.run_bzr(['foo', 'bar'], working_dir=None)
-        self.assertEqual(cwd, self.working_dir)
-
-        # The function should be run in the alternative directory
-        # but afterwards the current working dir shouldn't be changed
-        self.run_bzr(['foo', 'bar'], working_dir='one')
-        self.assertNotEqual(cwd, self.working_dir)
-        self.assertEndsWith(self.working_dir, 'one')
-        self.assertEqual(cwd, osutils.getcwd())
-
-        self.run_bzr(['foo', 'bar'], working_dir='two')
-        self.assertNotEqual(cwd, self.working_dir)
-        self.assertEndsWith(self.working_dir, 'two')
-        self.assertEqual(cwd, osutils.getcwd())
-
-
-class TestRunBzrSubprocess(TestCaseWithTransport):
-
-    def test_run_bzr_subprocess(self):
-        """The run_bzr_helper_external command behaves nicely."""
-        result = self.run_bzr_subprocess('--version')
-        result = self.run_bzr_subprocess(['--version'])
-        result = self.run_bzr_subprocess('--version', retcode=None)
-        self.assertContainsRe(result[0], 'is free software')
-        self.assertRaises(AssertionError, self.run_bzr_subprocess,
-                          '--versionn')
-        result = self.run_bzr_subprocess('--versionn', retcode=3)
-        result = self.run_bzr_subprocess('--versionn', retcode=None)
-        self.assertContainsRe(result[1], 'unknown command')
-        err = self.run_bzr_subprocess(['merge', '--merge-type',
-                                      'magic merge'], retcode=3)[1]
-        self.assertContainsRe(err, 'Bad value "magic merge" for option'
-                              ' "merge-type"')
-
-    def test_run_bzr_subprocess_env(self):
-        """run_bzr_subprocess can set environment variables in the child only.
-
-        These changes should not change the running process, only the child.
-        """
-        # The test suite should unset this variable
-        self.assertEqual(None, os.environ.get('BZR_EMAIL'))
-        out, err = self.run_bzr_subprocess('whoami', env_changes={
-                                            'BZR_EMAIL':'Joe Foo <joe@foo.com>'
-                                          }, universal_newlines=True)
-        self.assertEqual('', err)
-        self.assertEqual('Joe Foo <joe@foo.com>\n', out)
-        # And it should not be modified
-        self.assertEqual(None, os.environ.get('BZR_EMAIL'))
-
-        # Do it again with a different address, just to make sure
-        # it is actually changing
-        out, err = self.run_bzr_subprocess('whoami', env_changes={
-                                            'BZR_EMAIL':'Barry <bar@foo.com>'
-                                          }, universal_newlines=True)
-        self.assertEqual('', err)
-        self.assertEqual('Barry <bar@foo.com>\n', out)
-        self.assertEqual(None, os.environ.get('BZR_EMAIL'))
-
-    def test_run_bzr_subprocess_env_del(self):
-        """run_bzr_subprocess can remove environment variables too."""
-        # Create a random email, so we are sure this won't collide
-        rand_bzr_email = 'John Doe <jdoe@%s.com>' % (osutils.rand_chars(20),)
-        rand_email = 'Jane Doe <jdoe@%s.com>' % (osutils.rand_chars(20),)
-        os.environ['BZR_EMAIL'] = rand_bzr_email
-        os.environ['EMAIL'] = rand_email
-        try:
-            # By default, the child will inherit the current env setting
-            out, err = self.run_bzr_subprocess('whoami', universal_newlines=True)
-            self.assertEqual('', err)
-            self.assertEqual(rand_bzr_email + '\n', out)
-
-            # Now that BZR_EMAIL is not set, it should fall back to EMAIL
-            out, err = self.run_bzr_subprocess('whoami',
-                                               env_changes={'BZR_EMAIL':None},
-                                               universal_newlines=True)
-            self.assertEqual('', err)
-            self.assertEqual(rand_email + '\n', out)
-
-            # This switches back to the default email guessing logic
-            # Which shouldn't match either of the above addresses
-            out, err = self.run_bzr_subprocess('whoami',
-                           env_changes={'BZR_EMAIL':None, 'EMAIL':None},
-                           universal_newlines=True)
-
-            self.assertEqual('', err)
-            self.assertNotEqual(rand_bzr_email + '\n', out)
-            self.assertNotEqual(rand_email + '\n', out)
-        finally:
-            # TestCase cleans up BZR_EMAIL, and EMAIL at startup
-            del os.environ['BZR_EMAIL']
-            del os.environ['EMAIL']
-
-    def test_run_bzr_subprocess_env_del_missing(self):
-        """run_bzr_subprocess won't fail if deleting a nonexistant env var"""
-        self.failIf('NON_EXISTANT_ENV_VAR' in os.environ)
-        out, err = self.run_bzr_subprocess('rocks',
-                        env_changes={'NON_EXISTANT_ENV_VAR':None},
-                        universal_newlines=True)
-        self.assertEqual('It sure does!\n', out)
-        self.assertEqual('', err)
-
-    def test_run_bzr_subprocess_working_dir(self):
-        """Test that we can specify the working dir for the child"""
-        cwd = osutils.getcwd()
-
-        self.make_branch_and_tree('.')
-        self.make_branch_and_tree('one')
-        self.make_branch_and_tree('two')
-
-        def get_root(**kwargs):
-            """Spawn a process to get the 'root' of the tree.
-
-            You can pass in arbitrary new arguments. This just makes
-            sure that the returned path doesn't have trailing whitespace.
-            """
-            return self.run_bzr_subprocess('root', **kwargs)[0].rstrip()
-
-        self.assertEqual(cwd, get_root())
-        self.assertEqual(cwd, get_root(working_dir=None))
-        # Has our path changed?
-        self.assertEqual(cwd, osutils.getcwd())
-
-        dir1 = get_root(working_dir='one')
-        self.assertEndsWith(dir1, 'one')
-        self.assertEqual(cwd, osutils.getcwd())
-
-        dir2 = get_root(working_dir='two')
-        self.assertEndsWith(dir2, 'two')
-        self.assertEqual(cwd, osutils.getcwd())
-
-
-class _DontSpawnProcess(Exception):
-    """A simple exception which just allows us to skip unnecessary steps"""
-
-
-class TestRunBzrSubprocessCommands(TestCaseWithTransport):
-
-    def _popen(self, *args, **kwargs):
-        """Record the command that is run, so that we can ensure it is correct"""
-        self._popen_args = args
-        self._popen_kwargs = kwargs
-        raise _DontSpawnProcess()
-
-    def test_run_bzr_subprocess_no_plugins(self):
-        self.assertRaises(_DontSpawnProcess, self.run_bzr_subprocess, '')
-        command = self._popen_args[0]
-        self.assertEqual(sys.executable, command[0])
-        self.assertEqual(self.get_bzr_path(), command[1])
-        self.assertEqual(['--no-plugins'], command[2:])
-
-    def test_allow_plugins(self):
-        self.assertRaises(_DontSpawnProcess,
-                          self.run_bzr_subprocess, '', allow_plugins=True)
-        command = self._popen_args[0]
-        self.assertEqual([], command[2:])
-
-
-class TestBzrSubprocess(TestCaseWithTransport):
-
-    def test_start_and_stop_bzr_subprocess(self):
-        """We can start and perform other test actions while that process is
-        still alive.
-        """
-        process = self.start_bzr_subprocess(['--version'])
-        result = self.finish_bzr_subprocess(process)
-        self.assertContainsRe(result[0], 'is free software')
-        self.assertEqual('', result[1])
-
-    def test_start_and_stop_bzr_subprocess_with_error(self):
-        """finish_bzr_subprocess allows specification of the desired exit code.
-        """
-        process = self.start_bzr_subprocess(['--versionn'])
-        result = self.finish_bzr_subprocess(process, retcode=3)
-        self.assertEqual('', result[0])
-        self.assertContainsRe(result[1], 'unknown command')
-
-    def test_start_and_stop_bzr_subprocess_ignoring_retcode(self):
-        """finish_bzr_subprocess allows the exit code to be ignored."""
-        process = self.start_bzr_subprocess(['--versionn'])
-        result = self.finish_bzr_subprocess(process, retcode=None)
-        self.assertEqual('', result[0])
-        self.assertContainsRe(result[1], 'unknown command')
-
-    def test_start_and_stop_bzr_subprocess_with_unexpected_retcode(self):
-        """finish_bzr_subprocess raises self.failureException if the retcode is
-        not the expected one.
-        """
-        process = self.start_bzr_subprocess(['--versionn'])
-        self.assertRaises(self.failureException, self.finish_bzr_subprocess,
-                          process)
-
-    def test_start_and_stop_bzr_subprocess_send_signal(self):
-        """finish_bzr_subprocess raises self.failureException if the retcode is
-        not the expected one.
-        """
-        process = self.start_bzr_subprocess(['wait-until-signalled'],
-                                            skip_if_plan_to_signal=True)
-        self.assertEqual('running\n', process.stdout.readline())
-        result = self.finish_bzr_subprocess(process, send_signal=signal.SIGINT,
-                                            retcode=3)
-        self.assertEqual('', result[0])
-        self.assertEqual('bzr: interrupted\n', result[1])
-
-    def test_start_and_stop_working_dir(self):
-        cwd = osutils.getcwd()
-
-        self.make_branch_and_tree('one')
-
-        process = self.start_bzr_subprocess(['root'], working_dir='one')
-        result = self.finish_bzr_subprocess(process, universal_newlines=True)
-        self.assertEndsWith(result[0], 'one\n')
-        self.assertEqual('', result[1])
-
-
-class TestRunBzrError(ExternalBase):
-
-    def test_run_bzr_error(self):
-        # retcode=0 is specially needed here because run_bzr_error expects
-        # an error (oddly enough) but we want to test the case of not
-        # actually getting one
-        out, err = self.run_bzr_error(['^$'], ['rocks'], retcode=0)
-        self.assertEqual(out, 'It sure does!\n')
-        # now test actually getting an error
-        out, err = self.run_bzr_error(
-                ["bzr: ERROR: foobarbaz is not versioned"],
-                ['file-id', 'foobarbaz'])
-
-
-class TestSelftestListOnly(TestCase):
-
-    @staticmethod
-    def _parse_test_list(lines, newlines_in_header=0):
+        params = self.get_params_passed_to_core('selftest --subunit')
+        self.assertEqual(tests.SubUnitBzrRunner, params[1]['runner_class'])
+
+    def _parse_test_list(self, lines, newlines_in_header=0):
         "Parse a list of lines into a tuple of 3 lists (header,body,footer)."
         in_header = newlines_in_header != 0
         in_footer = False
@@ -512,92 +149,26 @@
         return (header,body,footer)
 
     def test_list_only(self):
-        # check that bzr selftest --list-only works correctly
-        out,err = self.run_bzr('selftest selftest --list-only')
-        (header,body,footer) = self._parse_test_list(out.splitlines())
-        num_tests = len(body)
-        self.assertLength(0, header)
-        self.assertLength(0, footer)
-        self.assertEqual('', err)
-
-    def test_list_only_filtered(self):
-        # check that a filtered --list-only works, both include and exclude
-        out_all,err_all = self.run_bzr('selftest --list-only')
-        tests_all = self._parse_test_list(out_all.splitlines())[1]
-        out_incl,err_incl = self.run_bzr('selftest --list-only selftest')
-        tests_incl = self._parse_test_list(out_incl.splitlines())[1]
-        self.assertSubset(tests_incl, tests_all)
-        out_excl,err_excl = self.run_bzr(['selftest', '--list-only',
-                                          '--exclude', 'selftest'])
-        tests_excl = self._parse_test_list(out_excl.splitlines())[1]
-        self.assertSubset(tests_excl, tests_all)
-        set_incl = set(tests_incl)
-        set_excl = set(tests_excl)
-        intersection = set_incl.intersection(set_excl)
-        self.assertEquals(0, len(intersection))
-        self.assertEquals(len(tests_all), len(tests_incl) + len(tests_excl))
-
-    def test_list_only_random(self):
-        # check that --randomize works correctly
-        out_all,err_all = self.run_bzr('selftest --list-only selftest')
-        tests_all = self._parse_test_list(out_all.splitlines())[1]
-        # XXX: It looks like there are some orders for generating tests that
-        # fail as of 20070504 - maybe because of import order dependencies.
-        # So unfortunately this will rarely intermittently fail at the moment.
-        # -- mbp 20070504
-        out_rand,err_rand = self.run_bzr(['selftest', '--list-only',
-                                          'selftest', '--randomize', 'now'])
-        (header_rand,tests_rand,dummy) = self._parse_test_list(
-            out_rand.splitlines(), 1)
-        # XXX: The following line asserts that the randomized order is not the
-        # same as the default order.  It is just possible that they'll get
-        # randomized into the same order and this will falsely fail, but
-        # that's very unlikely in practice because there are thousands of
-        # tests.
-        self.assertNotEqual(tests_all, tests_rand)
-        self.assertEqual(sorted(tests_all), sorted(tests_rand))
-        # Check that the seed can be reused to get the exact same order
-        seed_re = re.compile('Randomizing test order using seed (\w+)')
-        match_obj = seed_re.search(header_rand[-1])
-        seed = match_obj.group(1)
-        out_rand2,err_rand2 = self.run_bzr(['selftest', '--list-only',
-                                            'selftest', '--randomize', seed])
-        (header_rand2,tests_rand2,dummy) = self._parse_test_list(
-            out_rand2.splitlines(), 1)
-        self.assertEqual(tests_rand, tests_rand2)
-
-
-class TestSelftestWithIdList(TestCaseInTempDir):
-
-    def test_load_list(self):
-        # We don't want to call selftest for the whole suite, so we start with
-        # a reduced list.
-        test_list_fname = 'test.list'
-        fl = open(test_list_fname, 'wt')
-        fl.write('%s\n' % self.id())
-        fl.close()
-        out, err = self.run_bzr(
-            ['selftest', '--load-list', test_list_fname, '--list'])
-        self.assertContainsRe(out, "TestSelftestWithIdList")
-        self.assertLength(1, out.splitlines())
-
-    def test_load_unknown(self):
-        out, err = self.run_bzr('selftest --load-list I_do_not_exist ',
-                                retcode=3)
-
-
-class TestSelftestStartingWith(TestCase):
-
-    def test_starting_with_single_argument(self):
-        out, err = self.run_bzr(
-            ['selftest', '--starting-with', self.id(), '--list'])
-        self.assertContainsRe(out, self.id())
-
-    def test_starting_with_multiple_argument(self):
-        out, err = self.run_bzr(
-            ['selftest',
-             '--starting-with', self.id(),
-             '--starting-with', 'bzrlib.tests.test_sampler',
-             '--list'])
-        self.assertContainsRe(out, self.id())
-        self.assertContainsRe(out, 'bzrlib.tests.test_sampler')
+        # check that bzr selftest --list-only outputs no ui noise
+        def selftest(*args, **kwargs):
+            """Capture the arguments selftest was run with."""
+            return True
+        def outputs_nothing(cmdline):
+            out,err = self.run_bzr(cmdline)
+            (header,body,footer) = self._parse_test_list(out.splitlines())
+            num_tests = len(body)
+            self.assertLength(0, header)
+            self.assertLength(0, footer)
+            self.assertEqual('', err)
+        # Yes this prevents using threads to run the test suite in parallel,
+        # however we don't have a clean dependency injector for commands, 
+        # and even if we did - we'd still be testing that the glue is wired
+        # up correctly. XXX: TODO: Solve this testing problem.
+        original_selftest = tests.selftest
+        tests.selftest = selftest
+        try:
+            outputs_nothing('selftest --list-only')
+            outputs_nothing('selftest --list-only selftest')
+            outputs_nothing(['selftest', '--list-only', '--exclude', 'selftest'])
+        finally:
+            tests.selftest = original_selftest

=== modified file 'bzrlib/tests/per_tree/test_path_content_summary.py'
--- bzrlib/tests/per_tree/test_path_content_summary.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_tree/test_path_content_summary.py	2009-08-26 05:34:10 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2007 Canonical Ltd
+# Copyright (C) 2007, 2009 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -34,6 +34,18 @@
         self.addCleanup(result.unlock)
         return result
 
+    def check_content_summary_size(self, tree, summary, expected_size):
+        # if the tree supports content filters, then it's allowed to leave out
+        # the size because it might be difficult to compute.  otherwise, it
+        # must be present and correct
+        returned_size = summary[1]
+        if returned_size == expected_size or (
+            tree.supports_content_filtering()
+            and returned_size is None):
+            pass
+        else:
+            self.fail("invalid size in summary: %r" % (returned_size,))
+
     def test_symlink_content_summary(self):
         self.requireFeature(tests.SymlinkFeature)
         tree = self.make_branch_and_tree('tree')
@@ -76,8 +88,7 @@
         summary = self._convert_tree(tree).path_content_summary('path')
         self.assertEqual(4, len(summary))
         self.assertEqual('file', summary[0])
-        # size must be known
-        self.assertEqual(22, summary[1])
+        self.check_content_summary_size(tree, summary, 22)
         # executable
         self.assertEqual(True, summary[2])
         # may have hash,
@@ -91,8 +102,7 @@
         summary = self._convert_tree(tree).path_content_summary('path')
         self.assertEqual(4, len(summary))
         self.assertEqual('file', summary[0])
-        # size must be known
-        self.assertEqual(22, summary[1])
+        self.check_content_summary_size(tree, summary, 22)
         # not executable
         if osutils.supports_executable:
             self.assertEqual(False, summary[2])

=== modified file 'bzrlib/tests/per_workingtree/test_content_filters.py'
--- bzrlib/tests/per_workingtree/test_content_filters.py	2009-07-10 07:14:02 +0000
+++ bzrlib/tests/per_workingtree/test_content_filters.py	2009-08-25 04:43:21 +0000
@@ -43,6 +43,25 @@
     return _converter_helper(chunks, 'lower')
 
 
+_trailer_string = '\nend string\n'
+
+
+def _append_text(chunks, context=None):
+    """A content filter that appends a string to the end of the file.
+
+    This tests filters that change the length."""
+    return chunks + [_trailer_string]
+
+
+def _remove_appended_text(chunks, context=None):
+    """Remove the appended text."""
+
+    text = ''.join(chunks)
+    if text.endswith(_trailer_string):
+        text = text[:-len(_trailer_string)]
+    return [text]
+
+
 class TestWorkingTreeWithContentFilters(TestCaseWithWorkingTree):
 
     def create_cf_tree(self, txt_reader, txt_writer, dir='.'):
@@ -159,3 +178,34 @@
         self.assertFileEqual("fOO tXT", 'target/file1.txt')
         changes = target.changes_from(source.basis_tree())
         self.assertFalse(changes.has_changed())
+
+    def test_path_content_summary(self):
+        """path_content_summary should always talk about the canonical form."""
+        # see https://bugs.edge.launchpad.net/bzr/+bug/415508
+        #
+        # set up a tree where the canonical form has a string added to the
+        # end
+        source, txt_fileid, bin_fileid = self.create_cf_tree(
+            txt_reader=_append_text,
+            txt_writer=_remove_appended_text,
+            dir='source')
+        if not source.supports_content_filtering():
+            return
+        source.lock_read()
+        self.addCleanup(source.unlock)
+
+        expected_canonical_form = 'Foo Txt\nend string\n'
+        self.assertEquals(source.get_file(txt_fileid, filtered=True).read(),
+            expected_canonical_form)
+        self.assertEquals(source.get_file(txt_fileid, filtered=False).read(),
+            'Foo Txt')
+
+        # results are: kind, size, executable, sha1_or_link_target
+        result = source.path_content_summary('file1.txt')
+
+        self.assertEquals(result,
+            ('file', None, False, None))
+
+        # we could give back the length of the canonical form, but in general
+        # that will be expensive to compute, so it's acceptable to just return
+        # None.

=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2009-08-17 03:47:03 +0000
+++ bzrlib/tests/test_selftest.py	2009-08-24 05:35:28 +0000
@@ -18,6 +18,7 @@
 
 from cStringIO import StringIO
 import os
+import signal
 import sys
 import time
 import unittest
@@ -50,6 +51,7 @@
     deprecated_method,
     )
 from bzrlib.tests import (
+    SubUnitFeature,
     test_lsprof,
     test_sftp_transport,
     TestUtil,
@@ -1745,7 +1747,7 @@
 
     def test_make_tree_for_sftp_branch(self):
         """Transports backed by local directories create local trees."""
-
+        # NB: This is arguably a bug in the definition of make_branch_and_tree.
         tree = self.make_branch_and_tree('t1')
         base = tree.bzrdir.root_transport.base
         self.failIf(base.startswith('sftp'),
@@ -1756,7 +1758,24 @@
                 tree.branch.repository.bzrdir.root_transport)
 
 
-class TestSelftest(tests.TestCase):
+class SelfTestHelper:
+
+    def run_selftest(self, **kwargs):
+        """Run selftest returning its output."""
+        output = StringIO()
+        old_transport = bzrlib.tests.default_transport
+        old_root = tests.TestCaseWithMemoryTransport.TEST_ROOT
+        tests.TestCaseWithMemoryTransport.TEST_ROOT = None
+        try:
+            self.assertEqual(True, tests.selftest(stream=output, **kwargs))
+        finally:
+            bzrlib.tests.default_transport = old_transport
+            tests.TestCaseWithMemoryTransport.TEST_ROOT = old_root
+        output.seek(0)
+        return output
+
+
+class TestSelftest(tests.TestCase, SelfTestHelper):
     """Tests of bzrlib.tests.selftest."""
 
     def test_selftest_benchmark_parameter_invokes_test_suite__benchmark__(self):
@@ -1770,6 +1789,511 @@
             test_suite_factory=factory)
         self.assertEqual([True], factory_called)
 
+    def factory(self):
+        """A test suite factory."""
+        class Test(tests.TestCase):
+            def a(self):
+                pass
+            def b(self):
+                pass
+            def c(self):
+                pass
+        return TestUtil.TestSuite([Test("a"), Test("b"), Test("c")])
+
+    def test_list_only(self):
+        output = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True)
+        self.assertEqual(3, len(output.readlines()))
+
+    def test_list_only_filtered(self):
+        output = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, pattern="Test.b")
+        self.assertEndsWith(output.getvalue(), "Test.b\n")
+        self.assertLength(1, output.readlines())
+
+    def test_list_only_excludes(self):
+        output = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, exclude_pattern="Test.b")
+        self.assertNotContainsRe("Test.b", output.getvalue())
+        self.assertLength(2, output.readlines())
+
+    def test_random(self):
+        # test randomising by listing a number of tests.
+        output_123 = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, random_seed="123")
+        output_234 = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, random_seed="234")
+        self.assertNotEqual(output_123, output_234)
+        # "Randominzing test order..\n\n
+        self.assertLength(5, output_123.readlines())
+        self.assertLength(5, output_234.readlines())
+
+    def test_random_reuse_is_same_order(self):
+        # test randomising by listing a number of tests.
+        expected = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, random_seed="123")
+        repeated = self.run_selftest(test_suite_factory=self.factory,
+            list_only=True, random_seed="123")
+        self.assertEqual(expected.getvalue(), repeated.getvalue())
+
+    def test_runner_class(self):
+        self.requireFeature(SubUnitFeature)
+        from subunit import ProtocolTestCase
+        stream = self.run_selftest(runner_class=tests.SubUnitBzrRunner,
+            test_suite_factory=self.factory)
+        test = ProtocolTestCase(stream)
+        result = unittest.TestResult()
+        test.run(result)
+        self.assertEqual(3, result.testsRun)
+
+    def test_starting_with_single_argument(self):
+        output = self.run_selftest(test_suite_factory=self.factory,
+            starting_with=['bzrlib.tests.test_selftest.Test.a'],
+            list_only=True)
+        self.assertEqual('bzrlib.tests.test_selftest.Test.a\n',
+            output.getvalue())
+
+    def test_starting_with_multiple_argument(self):
+        output = self.run_selftest(test_suite_factory=self.factory,
+            starting_with=['bzrlib.tests.test_selftest.Test.a',
+                'bzrlib.tests.test_selftest.Test.b'],
+            list_only=True)
+        self.assertEqual('bzrlib.tests.test_selftest.Test.a\n'
+            'bzrlib.tests.test_selftest.Test.b\n',
+            output.getvalue())
+
+    def check_transport_set(self, transport_server):
+        captured_transport = []
+        def seen_transport(a_transport):
+            captured_transport.append(a_transport)
+        class Capture(tests.TestCase):
+            def a(self):
+                seen_transport(bzrlib.tests.default_transport)
+        def factory():
+            return TestUtil.TestSuite([Capture("a")])
+        self.run_selftest(transport=transport_server, test_suite_factory=factory)
+        self.assertEqual(transport_server, captured_transport[0])
+
+    def test_transport_sftp(self):
+        try:
+            import bzrlib.transport.sftp
+        except ParamikoNotPresent:
+            raise TestSkipped("Paramiko not present")
+        self.check_transport_set(bzrlib.transport.sftp.SFTPAbsoluteServer)
+
+    def test_transport_memory(self):
+        self.check_transport_set(bzrlib.transport.memory.MemoryServer)
+
+
+class TestSelftestWithIdList(tests.TestCaseInTempDir, SelfTestHelper):
+    # Does IO: reads test.list
+
+    def test_load_list(self):
+        # Provide a list with one test - this test.
+        test_id_line = '%s\n' % self.id()
+        self.build_tree_contents([('test.list', test_id_line)])
+        # And generate a list of the tests in  the suite.
+        stream = self.run_selftest(load_list='test.list', list_only=True)
+        self.assertEqual(test_id_line, stream.getvalue())
+
+    def test_load_unknown(self):
+        # Provide a list with one test - this test.
+        # And generate a list of the tests in  the suite.
+        err = self.assertRaises(errors.NoSuchFile, self.run_selftest,
+            load_list='missing file name', list_only=True)
+
+
+class TestRunBzr(tests.TestCase):
+
+    out = ''
+    err = ''
+
+    def _run_bzr_core(self, argv, retcode=0, encoding=None, stdin=None,
+                         working_dir=None):
+        """Override _run_bzr_core to test how it is invoked by run_bzr.
+
+        Attempts to run bzr from inside this class don't actually run it.
+
+        We test how run_bzr actually invokes bzr in another location.
+        Here we only need to test that it is run_bzr passes the right
+        parameters to run_bzr.
+        """
+        self.argv = list(argv)
+        self.retcode = retcode
+        self.encoding = encoding
+        self.stdin = stdin
+        self.working_dir = working_dir
+        return self.out, self.err
+
+    def test_run_bzr_error(self):
+        self.out = "It sure does!\n"
+        out, err = self.run_bzr_error(['^$'], ['rocks'], retcode=34)
+        self.assertEqual(['rocks'], self.argv)
+        self.assertEqual(34, self.retcode)
+        self.assertEqual(out, 'It sure does!\n')
+
+    def test_run_bzr_error_regexes(self):
+        self.out = ''
+        self.err = "bzr: ERROR: foobarbaz is not versioned"
+        out, err = self.run_bzr_error(
+                ["bzr: ERROR: foobarbaz is not versioned"],
+                ['file-id', 'foobarbaz'])
+
+    def test_encoding(self):
+        """Test that run_bzr passes encoding to _run_bzr_core"""
+        self.run_bzr('foo bar')
+        self.assertEqual(None, self.encoding)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr('foo bar', encoding='baz')
+        self.assertEqual('baz', self.encoding)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+    def test_retcode(self):
+        """Test that run_bzr passes retcode to _run_bzr_core"""
+        # Default is retcode == 0
+        self.run_bzr('foo bar')
+        self.assertEqual(0, self.retcode)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr('foo bar', retcode=1)
+        self.assertEqual(1, self.retcode)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr('foo bar', retcode=None)
+        self.assertEqual(None, self.retcode)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr(['foo', 'bar'], retcode=3)
+        self.assertEqual(3, self.retcode)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+    def test_stdin(self):
+        # test that the stdin keyword to run_bzr is passed through to
+        # _run_bzr_core as-is. We do this by overriding
+        # _run_bzr_core in this class, and then calling run_bzr,
+        # which is a convenience function for _run_bzr_core, so
+        # should invoke it.
+        self.run_bzr('foo bar', stdin='gam')
+        self.assertEqual('gam', self.stdin)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr('foo bar', stdin='zippy')
+        self.assertEqual('zippy', self.stdin)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+    def test_working_dir(self):
+        """Test that run_bzr passes working_dir to _run_bzr_core"""
+        self.run_bzr('foo bar')
+        self.assertEqual(None, self.working_dir)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+        self.run_bzr('foo bar', working_dir='baz')
+        self.assertEqual('baz', self.working_dir)
+        self.assertEqual(['foo', 'bar'], self.argv)
+
+    def test_reject_extra_keyword_arguments(self):
+        self.assertRaises(TypeError, self.run_bzr, "foo bar",
+                          error_regex=['error message'])
+
+
+class TestRunBzrCaptured(tests.TestCaseWithTransport):
+    # Does IO when testing the working_dir parameter.
+
+    def apply_redirected(self, stdin=None, stdout=None, stderr=None,
+                         a_callable=None, *args, **kwargs):
+        self.stdin = stdin
+        self.factory_stdin = getattr(bzrlib.ui.ui_factory, "stdin", None)
+        self.factory = bzrlib.ui.ui_factory
+        self.working_dir = osutils.getcwd()
+        stdout.write('foo\n')
+        stderr.write('bar\n')
+        return 0
+
+    def test_stdin(self):
+        # test that the stdin keyword to _run_bzr_core is passed through to
+        # apply_redirected as a StringIO. We do this by overriding
+        # apply_redirected in this class, and then calling _run_bzr_core,
+        # which calls apply_redirected.
+        self.run_bzr(['foo', 'bar'], stdin='gam')
+        self.assertEqual('gam', self.stdin.read())
+        self.assertTrue(self.stdin is self.factory_stdin)
+        self.run_bzr(['foo', 'bar'], stdin='zippy')
+        self.assertEqual('zippy', self.stdin.read())
+        self.assertTrue(self.stdin is self.factory_stdin)
+
+    def test_ui_factory(self):
+        # each invocation of self.run_bzr should get its
+        # own UI factory, which is an instance of TestUIFactory,
+        # with stdin, stdout and stderr attached to the stdin,
+        # stdout and stderr of the invoked run_bzr
+        current_factory = bzrlib.ui.ui_factory
+        self.run_bzr(['foo'])
+        self.failIf(current_factory is self.factory)
+        self.assertNotEqual(sys.stdout, self.factory.stdout)
+        self.assertNotEqual(sys.stderr, self.factory.stderr)
+        self.assertEqual('foo\n', self.factory.stdout.getvalue())
+        self.assertEqual('bar\n', self.factory.stderr.getvalue())
+        self.assertIsInstance(self.factory, tests.TestUIFactory)
+
+    def test_working_dir(self):
+        self.build_tree(['one/', 'two/'])
+        cwd = osutils.getcwd()
+
+        # Default is to work in the current directory
+        self.run_bzr(['foo', 'bar'])
+        self.assertEqual(cwd, self.working_dir)
+
+        self.run_bzr(['foo', 'bar'], working_dir=None)
+        self.assertEqual(cwd, self.working_dir)
+
+        # The function should be run in the alternative directory
+        # but afterwards the current working dir shouldn't be changed
+        self.run_bzr(['foo', 'bar'], working_dir='one')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'one')
+        self.assertEqual(cwd, osutils.getcwd())
+
+        self.run_bzr(['foo', 'bar'], working_dir='two')
+        self.assertNotEqual(cwd, self.working_dir)
+        self.assertEndsWith(self.working_dir, 'two')
+        self.assertEqual(cwd, osutils.getcwd())
+
+
+class StubProcess(object):
+    """A stub process for testing run_bzr_subprocess."""
+    
+    def __init__(self, out="", err="", retcode=0):
+        self.out = out
+        self.err = err
+        self.returncode = retcode
+
+    def communicate(self):
+        return self.out, self.err
+
+
+class TestRunBzrSubprocess(tests.TestCaseWithTransport):
+
+    def setUp(self):
+        tests.TestCaseWithTransport.setUp(self)
+        self.subprocess_calls = []
+
+    def start_bzr_subprocess(self, process_args, env_changes=None,
+                             skip_if_plan_to_signal=False,
+                             working_dir=None,
+                             allow_plugins=False):
+        """capture what run_bzr_subprocess tries to do."""
+        self.subprocess_calls.append({'process_args':process_args,
+            'env_changes':env_changes,
+            'skip_if_plan_to_signal':skip_if_plan_to_signal,
+            'working_dir':working_dir, 'allow_plugins':allow_plugins})
+        return self.next_subprocess
+
+    def assertRunBzrSubprocess(self, expected_args, process, *args, **kwargs):
+        """Run run_bzr_subprocess with args and kwargs using a stubbed process.
+
+        Inside TestRunBzrSubprocessCommands we use a stub start_bzr_subprocess
+        that will return static results. This assertion method populates those
+        results and also checks the arguments run_bzr_subprocess generates.
+        """
+        self.next_subprocess = process
+        try:
+            result = self.run_bzr_subprocess(*args, **kwargs)
+        except:
+            self.next_subprocess = None
+            for key, expected in expected_args.iteritems():
+                self.assertEqual(expected, self.subprocess_calls[-1][key])
+            raise
+        else:
+            self.next_subprocess = None
+            for key, expected in expected_args.iteritems():
+                self.assertEqual(expected, self.subprocess_calls[-1][key])
+            return result
+
+    def test_run_bzr_subprocess(self):
+        """The run_bzr_helper_external command behaves nicely."""
+        self.assertRunBzrSubprocess({'process_args':['--version']},
+            StubProcess(), '--version')
+        self.assertRunBzrSubprocess({'process_args':['--version']},
+            StubProcess(), ['--version'])
+        # retcode=None disables retcode checking
+        result = self.assertRunBzrSubprocess({},
+            StubProcess(retcode=3), '--version', retcode=None)
+        result = self.assertRunBzrSubprocess({},
+            StubProcess(out="is free software"), '--version')
+        self.assertContainsRe(result[0], 'is free software')
+        # Running a subcommand that is missing errors
+        self.assertRaises(AssertionError, self.assertRunBzrSubprocess,
+            {'process_args':['--versionn']}, StubProcess(retcode=3),
+            '--versionn')
+        # Unless it is told to expect the error from the subprocess
+        result = self.assertRunBzrSubprocess({},
+            StubProcess(retcode=3), '--versionn', retcode=3)
+        # Or to ignore retcode checking
+        result = self.assertRunBzrSubprocess({},
+            StubProcess(err="unknown command", retcode=3), '--versionn',
+            retcode=None)
+        self.assertContainsRe(result[1], 'unknown command')
+
+    def test_env_change_passes_through(self):
+        self.assertRunBzrSubprocess(
+            {'env_changes':{'new':'value', 'changed':'newvalue', 'deleted':None}},
+            StubProcess(), '',
+            env_changes={'new':'value', 'changed':'newvalue', 'deleted':None})
+
+    def test_no_working_dir_passed_as_None(self):
+        self.assertRunBzrSubprocess({'working_dir': None}, StubProcess(), '')
+
+    def test_no_working_dir_passed_through(self):
+        self.assertRunBzrSubprocess({'working_dir': 'dir'}, StubProcess(), '',
+            working_dir='dir')
+
+    def test_run_bzr_subprocess_no_plugins(self):
+        self.assertRunBzrSubprocess({'allow_plugins': False},
+            StubProcess(), '')
+
+    def test_allow_plugins(self):
+        self.assertRunBzrSubprocess({'allow_plugins': True},
+            StubProcess(), '', allow_plugins=True)
+
+
+class _DontSpawnProcess(Exception):
+    """A simple exception which just allows us to skip unnecessary steps"""
+
+
+class TestStartBzrSubProcess(tests.TestCase):
+
+    def check_popen_state(self):
+        """Replace to make assertions when popen is called."""
+
+    def _popen(self, *args, **kwargs):
+        """Record the command that is run, so that we can ensure it is correct"""
+        self.check_popen_state()
+        self._popen_args = args
+        self._popen_kwargs = kwargs
+        raise _DontSpawnProcess()
+
+    def test_run_bzr_subprocess_no_plugins(self):
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [])
+        command = self._popen_args[0]
+        self.assertEqual(sys.executable, command[0])
+        self.assertEqual(self.get_bzr_path(), command[1])
+        self.assertEqual(['--no-plugins'], command[2:])
+
+    def test_allow_plugins(self):
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+            allow_plugins=True)
+        command = self._popen_args[0]
+        self.assertEqual([], command[2:])
+
+    def test_set_env(self):
+        self.failIf('EXISTANT_ENV_VAR' in os.environ)
+        # set in the child
+        def check_environment():
+            self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
+        self.check_popen_state = check_environment
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+            env_changes={'EXISTANT_ENV_VAR':'set variable'})
+        # not set in theparent
+        self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
+
+    def test_run_bzr_subprocess_env_del(self):
+        """run_bzr_subprocess can remove environment variables too."""
+        self.failIf('EXISTANT_ENV_VAR' in os.environ)
+        def check_environment():
+            self.assertFalse('EXISTANT_ENV_VAR' in os.environ)
+        os.environ['EXISTANT_ENV_VAR'] = 'set variable'
+        self.check_popen_state = check_environment
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+            env_changes={'EXISTANT_ENV_VAR':None})
+        # Still set in parent
+        self.assertEqual('set variable', os.environ['EXISTANT_ENV_VAR'])
+        del os.environ['EXISTANT_ENV_VAR']
+
+    def test_env_del_missing(self):
+        self.failIf('NON_EXISTANT_ENV_VAR' in os.environ)
+        def check_environment():
+            self.assertFalse('NON_EXISTANT_ENV_VAR' in os.environ)
+        self.check_popen_state = check_environment
+        self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+            env_changes={'NON_EXISTANT_ENV_VAR':None})
+
+    def test_working_dir(self):
+        """Test that we can specify the working dir for the child"""
+        orig_getcwd = osutils.getcwd
+        orig_chdir = os.chdir
+        chdirs = []
+        def chdir(path):
+            chdirs.append(path)
+        os.chdir = chdir
+        try:
+            def getcwd():
+                return 'current'
+            osutils.getcwd = getcwd
+            try:
+                self.assertRaises(_DontSpawnProcess, self.start_bzr_subprocess, [],
+                    working_dir='foo')
+            finally:
+                osutils.getcwd = orig_getcwd
+        finally:
+            os.chdir = orig_chdir
+        self.assertEqual(['foo', 'current'], chdirs)
+
+
+class TestBzrSubprocess(tests.TestCaseWithTransport):
+
+    def test_start_and_stop_bzr_subprocess(self):
+        """We can start and perform other test actions while that process is
+        still alive.
+        """
+        process = self.start_bzr_subprocess(['--version'])
+        result = self.finish_bzr_subprocess(process)
+        self.assertContainsRe(result[0], 'is free software')
+        self.assertEqual('', result[1])
+
+    def test_start_and_stop_bzr_subprocess_with_error(self):
+        """finish_bzr_subprocess allows specification of the desired exit code.
+        """
+        process = self.start_bzr_subprocess(['--versionn'])
+        result = self.finish_bzr_subprocess(process, retcode=3)
+        self.assertEqual('', result[0])
+        self.assertContainsRe(result[1], 'unknown command')
+
+    def test_start_and_stop_bzr_subprocess_ignoring_retcode(self):
+        """finish_bzr_subprocess allows the exit code to be ignored."""
+        process = self.start_bzr_subprocess(['--versionn'])
+        result = self.finish_bzr_subprocess(process, retcode=None)
+        self.assertEqual('', result[0])
+        self.assertContainsRe(result[1], 'unknown command')
+
+    def test_start_and_stop_bzr_subprocess_with_unexpected_retcode(self):
+        """finish_bzr_subprocess raises self.failureException if the retcode is
+        not the expected one.
+        """
+        process = self.start_bzr_subprocess(['--versionn'])
+        self.assertRaises(self.failureException, self.finish_bzr_subprocess,
+                          process)
+
+    def test_start_and_stop_bzr_subprocess_send_signal(self):
+        """finish_bzr_subprocess raises self.failureException if the retcode is
+        not the expected one.
+        """
+        process = self.start_bzr_subprocess(['wait-until-signalled'],
+                                            skip_if_plan_to_signal=True)
+        self.assertEqual('running\n', process.stdout.readline())
+        result = self.finish_bzr_subprocess(process, send_signal=signal.SIGINT,
+                                            retcode=3)
+        self.assertEqual('', result[0])
+        self.assertEqual('bzr: interrupted\n', result[1])
+
+    def test_start_and_stop_working_dir(self):
+        cwd = osutils.getcwd()
+        self.make_branch_and_tree('one')
+        process = self.start_bzr_subprocess(['root'], working_dir='one')
+        result = self.finish_bzr_subprocess(process, universal_newlines=True)
+        self.assertEndsWith(result[0], 'one\n')
+        self.assertEqual('', result[1])
+
 
 class TestKnownFailure(tests.TestCase):
 
@@ -1840,8 +2364,8 @@
         tests.TestCase.setUp(self)
         self.suite = TestUtil.TestSuite()
         self.loader = TestUtil.TestLoader()
-        self.suite.addTest(self.loader.loadTestsFromModuleNames([
-            'bzrlib.tests.test_selftest']))
+        self.suite.addTest(self.loader.loadTestsFromModule(
+            sys.modules['bzrlib.tests.test_selftest']))
         self.all_names = _test_ids(self.suite)
 
     def test_condition_id_re(self):
@@ -2158,8 +2682,8 @@
 class TestTestSuite(tests.TestCase):
 
     def test_test_suite(self):
-        # This test is slow, so we do a single test with one test in each
-        # category
+        # This test is slow - it loads the entire test suite to operate, so we
+        # do a single test with one test in each category
         test_list = [
             # testmod_names
             'bzrlib.tests.blackbox.test_branch.TestBranch.test_branch',
@@ -2175,6 +2699,9 @@
         self.assertEquals(test_list, _test_ids(suite))
 
     def test_test_suite_list_and_start(self):
+        # We cannot test this at the same time as the main load, because we want
+        # to know that starting_with == None works. So a second full load is
+        # incurred.
         test_list = ['bzrlib.tests.test_selftest.TestTestSuite.test_test_suite']
         suite = tests.test_suite(test_list,
                                  ['bzrlib.tests.test_selftest.TestTestSuite'])

=== modified file 'bzrlib/tests/test_shelf.py'
--- bzrlib/tests/test_shelf.py	2009-08-26 03:20:32 +0000
+++ bzrlib/tests/test_shelf.py	2009-08-26 07:40:38 +0000
@@ -473,6 +473,13 @@
         self.addCleanup(creator.finalize)
         self.assertEqual([], list(creator.iter_shelvable()))
 
+    def test_shelve_skips_added_root(self):
+        """Skip adds of the root when iterating through shelvable changes."""
+        tree = self.make_branch_and_tree('tree')
+        creator = shelf.ShelfCreator(tree, tree.basis_tree())
+        self.addCleanup(creator.finalize)
+        self.assertEqual([], list(creator.iter_shelvable()))
+
 
 class TestUnshelver(tests.TestCaseWithTransport):
 

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2009-08-26 03:20:32 +0000
+++ bzrlib/tree.py	2009-08-26 05:38:16 +0000
@@ -222,10 +222,14 @@
     def path_content_summary(self, path):
         """Get a summary of the information about path.
 
+        All the attributes returned are for the canonical form, not the
+        convenient form (if content filters are in use.)
+
         :param path: A relative path within the tree.
         :return: A tuple containing kind, size, exec, sha1-or-link.
             Kind is always present (see tree.kind()).
-            size is present if kind is file, None otherwise.
+            size is present if kind is file and the size of the 
+                canonical form can be cheaply determined, None otherwise.
             exec is None unless kind is file and the platform supports the 'x'
                 bit.
             sha1-or-link is the link target if kind is symlink, or the sha1 if

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2009-08-14 02:43:55 +0000
+++ bzrlib/workingtree.py	2009-08-26 05:38:16 +0000
@@ -613,6 +613,8 @@
 
     def get_file_size(self, file_id):
         """See Tree.get_file_size"""
+        # XXX: this returns the on-disk size; it should probably return the
+        # canonical size
         try:
             return os.path.getsize(self.id2abspath(file_id))
         except OSError, e:
@@ -749,11 +751,7 @@
             raise
         kind = _mapper(stat_result.st_mode)
         if kind == 'file':
-            size = stat_result.st_size
-            # try for a stat cache lookup
-            executable = self._is_executable_from_path_and_stat(path, stat_result)
-            return (kind, size, executable, self._sha_from_stat(
-                path, stat_result))
+            return self._file_content_summary(path, stat_result)
         elif kind == 'directory':
             # perhaps it looks like a plain directory, but it's really a
             # reference.
@@ -766,6 +764,13 @@
         else:
             return (kind, None, None, None)
 
+    def _file_content_summary(self, path, stat_result):
+        size = stat_result.st_size
+        executable = self._is_executable_from_path_and_stat(path, stat_result)
+        # try for a stat cache lookup
+        return ('file', size, executable, self._sha_from_stat(
+            path, stat_result))
+
     def _check_parents_for_ghosts(self, revision_ids, allow_leftmost_as_ghost):
         """Common ghost checking functionality from set_parent_*.
 

=== modified file 'bzrlib/workingtree_4.py'
--- bzrlib/workingtree_4.py	2009-08-04 11:25:23 +0000
+++ bzrlib/workingtree_4.py	2009-08-25 04:43:21 +0000
@@ -1300,6 +1300,32 @@
         return statvalue, sha1
 
 
+class ContentFilteringDirStateWorkingTree(DirStateWorkingTree):
+    """Dirstate working tree that supports content filtering.
+
+    The dirstate holds the hash and size of the canonical form of the file, 
+    and most methods must return that.
+    """
+
+    def _file_content_summary(self, path, stat_result):
+        # This is to support the somewhat obsolete path_content_summary method
+        # with content filtering: see
+        # <https://bugs.edge.launchpad.net/bzr/+bug/415508>.
+        #
+        # If the dirstate cache is up to date and knows the hash and size,
+        # return that.
+        # Otherwise if there are no content filters, return the on-disk size
+        # and leave the hash blank.
+        # Otherwise, read and filter the on-disk file and use its size and
+        # hash.
+        #
+        # The dirstate doesn't store the size of the canonical form so we
+        # can't trust it for content-filtered trees.  We just return None.
+        dirstate_sha1 = self._dirstate.sha1_from_stat(path, stat_result)
+        executable = self._is_executable_from_path_and_stat(path, stat_result)
+        return ('file', None, executable, dirstate_sha1)
+
+
 class WorkingTree4(DirStateWorkingTree):
     """This is the Format 4 working tree.
 
@@ -1313,7 +1339,7 @@
     """
 
 
-class WorkingTree5(DirStateWorkingTree):
+class WorkingTree5(ContentFilteringDirStateWorkingTree):
     """This is the Format 5 working tree.
 
     This differs from WorkingTree4 by:
@@ -1323,7 +1349,7 @@
     """
 
 
-class WorkingTree6(DirStateWorkingTree):
+class WorkingTree6(ContentFilteringDirStateWorkingTree):
     """This is the Format 6 working tree.
 
     This differs from WorkingTree5 by:
@@ -1550,7 +1576,11 @@
 
 
 class DirStateRevisionTree(Tree):
-    """A revision tree pulling the inventory from a dirstate."""
+    """A revision tree pulling the inventory from a dirstate.
+    
+    Note that this is one of the historical (ie revision) trees cached in the
+    dirstate for easy access, not the workingtree.
+    """
 
     def __init__(self, dirstate, revision_id, repository):
         self._dirstate = dirstate

=== modified file 'doc/developers/bug-handling.txt'
--- doc/developers/bug-handling.txt	2009-07-17 02:01:07 +0000
+++ doc/developers/bug-handling.txt	2009-08-24 00:29:31 +0000
@@ -142,8 +142,12 @@
     it's not a good idea for a developer to spend time reproducing the bug
     until they're going to work on it.)
 Triaged
-    We don't use this status.  If it is set, it means the same as
-    Confirmed.
+    This is an odd state - one we consider a bug in launchpad, as it really
+    means "Importance has been set". We use this to mean the same thing
+    as confirmed, and set no preference on whether Confirmed or Triaged are
+    used. Please do not change a "Confirmed" bug to "Triaged" or vice verca -
+    any reports we create or use will always search for both "Confirmed" and
+    "Triaged" or neither "Confirmed" nor "Triaged".
 In Progress
     Someone has started working on this.
 Won't Fix

